package service

import (
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/error_process"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/gorm_helper"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/model"
	bgm "git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/util"
	"github.com/gofiber/fiber/v2"
	"gorm.io/gorm"
)

// CreateTag 新建Tag(暂不支持创建连带Category)
func CreateTag(tag *model.Tag, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.CreateTagM(tag, db)
}

func UpdateTag(tag *model.Tag, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.UpdateTagM(tag, db)
}

func GetTag(page, limit int, ctx *fiber.Ctx, query bgm.QueryMap) ([]model.Tag, int64, error) {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return nil, 0, err
	}
	return model.GetTagM(page, limit, query, db)
}

func DeleteTag(tag *model.Tag, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	tag.IsDel = 1
	//根据uuid对 is_del字段update，其他字段不允许传入
	if tag.Value != "" {
		return error_process.ExtraValueErr("Value")
	}
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.UpdateTagM(tag, db)
}

// CreateCategoryForTag 新增删除Category分组
func CreateCategoryForTag(cTags *model.TagCategoriesDto, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.CreateCategoryForTagM(cTags.Tid, cTags.CIds, db)
}

// DelCategoryForTag 删除Category分组
func DelCategoryForTag(cTags *model.TagCategoriesDto, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.DelCategoryForTagM(cTags.Tid, cTags.CIds, db)
}
